<?php

namespace App\Http\Controllers;

use App\Models\Comment;
use App\Models\Faq;
use App\Models\Unit;
use App\Models\Post;
use App\Models\PostCategory;
use App\Models\Ticket;
use App\Models\Topic;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use \Cviebrock\EloquentSluggable\Services\SlugService;
use GuzzleHttp\Client;

class PostController extends Controller
{
    public function index()
    {
        $urlApi = config('api.api_url');
        $client = new Client([
            'headers' => [
                'Authorization' => 'Bearer ' . session('api_token'),
                'Accept' => 'application/json',
            ],
        ]);

        $search = request('search');
        $categorySearch = request('category');
        $orderSearch = request('order_by');
        $topicSearch = request('topic');

        $queryParams = [];
        if ($search) {
            $queryParams['search'] = $search;
        } elseif ($categorySearch) {
            $queryParams['category'] = $categorySearch;
        } elseif ($orderSearch) {
            $queryParams['order_by'] = $orderSearch;
        } elseif ($topicSearch) {
            $queryParams['topic'] = $topicSearch;
        }

        $responsePosts = $client->request('GET', $urlApi . 'posts', ['query' => $queryParams]);
        $statusCodeTickets = $responsePosts->getStatusCode();

        $responseCategories = $client->request('GET', $urlApi . 'categories');
        $statusCodeCategories = $responseCategories->getStatusCode();

        $responseTopics = $client->request('GET', $urlApi . 'topics');
        $statusCodeTopics = $responseTopics->getStatusCode();

        if ($statusCodeTickets == 200 && $statusCodeTopics == 200 && $statusCodeCategories == 200) {
            $posts = json_decode($responsePosts->getBody());
            $post_categories = json_decode($responseCategories->getBody());
            $topics = json_decode($responseTopics->getBody());

            return view('forum.user.index', [
                'title' => 'Forum Diskusi',
                'topics' => $topics->topics,
                'post_categories' => $post_categories->categories,
                'posts' => $posts->posts
            ]);
        } else {
            return view('errors.500', [
                'message' => '500 Internal Server Error'
            ]);
        }
    }


    public function show(Post $post)
    {
        $urlApi = config('api.api_url');

        $post_comments = Comment::latest()->where('post_id', $post->id)->whereNull('parent_id')->get();
        
        $client = new Client([
            'headers' => [
                'Authorization' => 'Bearer ' . session('api_token'),
                'Accept' => 'application/json',
            ],
        ]);

        $responsePosts = $client->request('GET', $urlApi . 'posts/' . $post->slug);
        $statusCode1 = $responsePosts->getStatusCode();

        if($statusCode1 == 200) {
            $post = json_decode($responsePosts->getBody());

            return view('forum.user.show', [
                'post' => $post->posts,
                'post_comments' => $post_comments,
                'title' => 'Detail Diskusi'
            ]);
        } else {
            return view('errors.500', [
                'message' => '500 Internal Server Error'
            ]);
        }
    }

    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'title' => 'required|max:255',
            'slug' => 'required',
            'topic_id' => 'required',
            'category_id' => 'required',
            'attachment' => 'image|file|max:2048',
            'body' => 'required'
        ]);

        $urlApi = config('api.api_url');

        $client = new Client();
    
        $response = $client->post($urlApi . 'posts', [
            'headers' => [
                'Authorization' => 'Bearer ' . session('api_token'),
                'Accept' => 'application/json',
            ],
            'json' => $validatedData
        ]);

        $statusCode = $response->getStatusCode();

        if($statusCode == 200) {
            return back()->with('toast_success', "Ticket berhasil dibuat");
        } else {
            return back()->with('toast_error', "Ticket gagal dibuat");
        }
    }

    public function update(Post $post)
    {
        $urlApi = config('api.api_url');

        $isOwner = $post->user_id == auth()->user()->id;
        $isAdmin = $post->unit_id == auth()->user()->unit_id && auth()->user()->role_id == 2;
 
        if($isOwner || $isAdmin) {
            $client = new Client();
    
            $response = $client->patch($urlApi . 'posts/' . $post->slug, [
                'headers' => [
                    'Authorization' => 'Bearer ' . session('api_token'),
                    'Accept' => 'application/json',
                ]
            ]);
    
            $statusCode = $response->getStatusCode();

            if($statusCode == 200) {
                return back()->with('toast_success', "Diskusi berhasil diselesaikan");
            } else {
                return back()->with('toast_error', "Diskusi gagal diselesaikan");
            }
        } else {
            return back()->with('toast_error', "Anda bukan pemilik tiket!");
        }
    }

    public function checkSlug(Request $request)
    {
        $slug = SlugService::createSlug(Post::class, 'slug', $request->title);

        return response()->json(['slug' => $slug]);
    }

    public function indexAdmin() {
        $urlApi = config('api.api_url');

        $client = new Client([
            'headers' => [
                'Authorization' => 'Bearer ' . session('api_token'),
                'Accept' => 'application/json',
            ],
        ]);

        $search = request('search');
        $categorySearch = request('category');
        $orderSearch = request('order_by');
        $topicSearch = request('topic');

        if($search) {            
            $responsePosts = $client->request('GET', $urlApi . 'posts?search=' . $search);
        } else if($categorySearch) {
            $responsePosts = $client->request('GET', $urlApi . 'posts?category=' . $categorySearch);
        } else if($orderSearch) {
            $responsePosts = $client->request('GET', $urlApi . 'posts?order_by=' . $orderSearch);
        } else if($topicSearch) {
            $responsePosts = $client->request('GET', $urlApi . 'posts?topic=' . $topicSearch);
        } else {
            $responsePosts = $client->request('GET', $urlApi . 'posts');
        }
        $statusCodeTickets = $responsePosts->getStatusCode();
            
        $responseCategories = $client->request('GET', $urlApi . 'categories');
        $statusCodeCategories = $responseCategories->getStatusCode();

        $responseTopics = $client->request('GET', $urlApi . 'topics');
        $statusCodeTopics = $responseCategories->getStatusCode();


        if($statusCodeTickets == 200 && $statusCodeTopics == 200 && $statusCodeCategories == 200) {
            $posts = json_decode($responsePosts->getBody());
            $post_categories = json_decode($responseCategories->getBody());
            $topics = json_decode($responseTopics->getBody());
    
            return view('forum.admin.index', [
                'title' => 'Forum Diskusi',
                'topics' => $topics->topics,
                'post_categories' => $post_categories->categories,
                'posts' => $posts->posts
            ]);
        } else {
            return view('errors.500', [
                'message' => '500 Internal Server Error'
            ]);
        }
    }

    public function showAdmin(Post $post)
    {
        $urlApi = config('api.api_url');

        $post_comments = Comment::latest()->where('post_id', $post->id)->whereNull('parent_id')->get();
        
        $client = new Client([
            'headers' => [
                'Authorization' => 'Bearer ' . session('api_token'),
                'Accept' => 'application/json',
            ],
        ]);

        $responsePosts = $client->request('GET', $urlApi . 'posts/' . $post->slug);
        $statusCode1 = $responsePosts->getStatusCode();

        if($statusCode1 == 200) {
            $post = json_decode($responsePosts->getBody());

            // return response()->json($post);

            return response()->json($post);

            return view('forum.admin.show', [
                'post' => $post->posts,
                'post_comments' => $post_comments,
                'title' => 'Detail Diskusi'
            ]);
        } else {
            return view('errors.500', [
                'message' => '500 Internal Server Error'
            ]);
        }
    }
}