<?php

namespace App\Http\Controllers;

use App\Models\Unit;
use App\Models\User;
use App\Models\UserDetail;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;
use Illuminate\Validation\Rule;

class UserController extends Controller
{
    public function index()
    {
        $urlApi = config('api.api_url');
        
        $client = new Client([
            'headers' => [
                'Authorization' => 'Bearer ' . session('api_token'),
                'Accept' => 'application/json',
            ],
        ]);
        
        $response = $client->request('GET', $urlApi . 'users');
            
        $statusCode = $response->getStatusCode();

        if($statusCode == 200) {
            $users = json_decode($response->getBody());

            return view('users.admin.index', [
                'title' => 'Manajemen User',
                'users' => $users->users
            ]);
        } else {
            return view('errors.500', [
                'message' => '500 Internal Server Error'
            ]);
        }
    }

    public function create()
    {
        return view('users.admin.create', [
            'title' => 'Tambah User'
        ]);
    }

    public function store(Request $request)
    {
        $urlApi = config('api.api_url');

        $validatedData = $request->validate([
            'nip' => 'required',
            'name' => 'required|max:255',
            'email' => 'required|email:dns|unique:users',
            'password' => 'required|min:8',
        ]);

        $client = new Client();
    
        $response = $client->post($urlApi . 'users', [
            'headers' => [
                'Authorization' => 'Bearer ' . session('api_token'),
                'Accept' => 'application/json',
            ],
            'json' => $validatedData
        ]);

        $statusCode = $response->getStatusCode();

        if($statusCode == 200) {
            return redirect('/dashboard/users')->with('toast_success', "User berhasil dibuat");
        } else {
            return redirect('/dashboard/users')->with('toast_error', "User gagal dibuat");
        }
    }

    public function edit(User $user)
    {
        return view('users.admin.edit', [
            'title' => 'Edit User',
            'user' => $user
        ]);
    }

    public function update(Request $request, User $user)
    {
        $urlApi = config('api.api_url');

        $validatedData = $request->validate([
            'nip' => 'required',
            'name' => 'required|max:255',
            'email' => [
                'required',
                'email:dns',
                Rule::unique('users')->ignore($user->id),
            ],
        ]);    

        $client = new Client();
    
        $response = $client->put($urlApi . 'users/' . $user->id, [
            'headers' => [
                'Authorization' => 'Bearer ' . session('api_token'),
                'Accept' => 'application/json',
            ],
            'json' => $validatedData
        ]);

        $statusCode = $response->getStatusCode();

        if($statusCode == 200) {
            return redirect('/dashboard/users')->with('toast_success', "User berhasil diupdate");
        } else {
            return redirect('/dashboard/users')->with('toast_error', "User gagal diupdate");
        }   
    }
    
    public function destroy($id)
    {
        $urlApi = config('api.api_url');

        $client = new Client();
    
        $response = $client->delete($urlApi . 'users/' . $id, [
            'headers' => [
                'Authorization' => 'Bearer ' . session('api_token'),
                'Accept' => 'application/json',
            ],
        ]);

        $statusCode = $response->getStatusCode();

        if($statusCode == 200) {
            return back()->with('toast_success', "User berhasil dihapus");
        } else {
            return back()->with('toast_error', "User gagal dihapus");
        }
    }

    public function generatePassword(Request $request)
    {
        $password = Hash::make(Str::random(10));

        return response()->json(['password' => $password]);
    }
}